/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal.introspection.details.handler;

import org.conqat.ide.commons.ui.jface.TreeContentProviderBase;
import org.eclipse.core.resources.IFile;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TableLabelProviderBase;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.ui.dialogs.PatternFilter;
import org.fortiss.tooling.kernel.extension.IEclipseResourcePostLoadProvider;
import org.fortiss.tooling.kernel.extension.IEclipseResourceStorageLocationProvider;
import org.fortiss.tooling.kernel.extension.data.ModelStorageError;
import org.fortiss.tooling.kernel.introspection.items.EclipseResourceStorageProviderIntrospectionDetailsItem;

/**
 * Introspection UI handler for the {@link EclipseResourceStorageProviderIntrospectionDetailsItem}s.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public final class EclipseResourceStorageProviderIntrospectionDetailsUIHandler extends
		IntrospectionDetailsUIHandlerBase {

	/** {@inheritDoc} */
	@Override
	public Control createComposite(ScrolledComposite parent) {
		return createFilteredTree(parent, "Type to search registered persistency providers:", null);
	}

	/** {@inheritDoc} */
	@Override
	protected void createTreeColumns(Tree tree) {
		TreeColumn col0 = new TreeColumn(tree, SWT.LEFT);
		col0.setText("Category / Provider / File");
		col0.setWidth(450);

		TreeColumn col1 = new TreeColumn(tree, SWT.LEFT);
		col1.setText("Implementation Class / Path / Storage Error Message");
		col1.setWidth(400);
	}

	/** String array of root categories. */
	private final String[] ROOT_CATEGORY = new String[] {"Storage Provider", "Post-load Provider",
			"Loaded Files", "Erroneous Files"};

	/** {@inheritDoc} */
	@Override
	protected ITreeContentProvider createContentProvider() {
		return new TreeContentProviderBase() {

			@Override
			public Object[] getChildren(Object parentElement) {
				if(parentElement == getInputObject()) {
					return ROOT_CATEGORY;
				}
				if(ROOT_CATEGORY[0].equals(parentElement)) {
					return getInputObject().getStorageProviderList().toArray();
				}
				if(ROOT_CATEGORY[1].equals(parentElement)) {
					return getInputObject().getPostLoadProviderList().toArray();
				}
				if(ROOT_CATEGORY[2].equals(parentElement)) {
					return getInputObject().getLoadedFiles().toArray();
				}
				if(ROOT_CATEGORY[3].equals(parentElement)) {
					return getInputObject().getErrorFiles().toArray();
				}
				return null;
			}
		};
	}

	/** {@inheritDoc} */
	@Override
	protected ITableLabelProvider createLabelProvider() {
		return new TableLabelProviderBase() {
			@Override
			public String getColumnText(Object parentElement, int columnIndex) {
				if(parentElement instanceof IEclipseResourceStorageLocationProvider ||
						parentElement instanceof IEclipseResourcePostLoadProvider) {
					return columnIndex == 0 ? parentElement.getClass().getSimpleName()
							: parentElement.getClass().getName();
				}
				if(parentElement instanceof String) {
					return columnIndex == 0 ? (String)parentElement : "";
				}
				if(parentElement instanceof IFile) {
					IFile file = (IFile)parentElement;
					return columnIndex == 0 ? file.getName() : file.getFullPath().toString();
				}
				if(parentElement instanceof ModelStorageError) {
					ModelStorageError error = (ModelStorageError)parentElement;
					return columnIndex == 0 ? "" : error.getMessage();
				}
				return "";
			}
		};
	}

	/** {@inheritDoc} */
	@Override
	protected PatternFilter createPatternFilter() {
		return new TableViewerPatternFilter();
	}

	/** {@inheritDoc} */
	@Override
	protected EclipseResourceStorageProviderIntrospectionDetailsItem getInputObject() {
		return (EclipseResourceStorageProviderIntrospectionDetailsItem)dataItem;
	}
}
