/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal.introspection.details.handler;

import org.conqat.ide.commons.ui.jface.TreeContentProviderBase;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TableLabelProviderBase;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.ui.dialogs.PatternFilter;
import org.fortiss.tooling.kernel.extension.IPrototypeProvider;
import org.fortiss.tooling.kernel.extension.data.Prototype;
import org.fortiss.tooling.kernel.extension.data.PrototypeCategory;
import org.fortiss.tooling.kernel.service.IPrototypeService;

/**
 * Introspection UI handler for the {@link IPrototypeService}.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public final class PrototypeServiceIntrospectionDetailsUIHandler extends
		IntrospectionDetailsUIHandlerBase {

	/** {@inheritDoc} */
	@Override
	public Control createComposite(ScrolledComposite parent) {
		String heading = "Type to search registered providers:";
		String footer =
				"Number of currently registered prototypes: " +
						IPrototypeService.INSTANCE.getAllPrototypes().size() + " (" +
						IPrototypeService.INSTANCE.getPrimaryPrototypes().size() +
						" primary prototypes)";
		return createFilteredTreeInTabFolder(parent, heading, footer, "Registered Providers");
	}

	/** Find the prototype provider for the given prototype. */
	private String findProtoypeProviderClassName(Prototype proto) {
		for(IPrototypeProvider provider : IPrototypeService.INSTANCE.getPrototypeProviders()) {
			if(provider.getPrototypes().contains(proto)) {
				return provider.getClass().getName();
			}
		}
		return "Unknown Prototype Provider!";
	}

	/** {@inheritDoc} */
	@Override
	protected void createTreeColumns(Tree tree) {
		TreeColumn col0 = new TreeColumn(tree, SWT.LEFT);
		col0.setText("Category / Prototype");
		col0.setWidth(200);

		TreeColumn col1 = new TreeColumn(tree, SWT.LEFT);
		col1.setText("Provider Class");
		col1.setWidth(400);

		TreeColumn col2 = new TreeColumn(tree, SWT.LEFT);
		col2.setText("EObject Class");
		col2.setWidth(400);

		TreeColumn col3 = new TreeColumn(tree, SWT.CENTER);
		col3.setText("Primary");
		col3.setWidth(50);
	}

	/** {@inheritDoc} */
	@Override
	protected ITreeContentProvider createContentProvider() {
		return new TreeContentProviderBase() {

			@Override
			public Object[] getChildren(Object parentElement) {
				if(parentElement instanceof IPrototypeService) {
					IPrototypeService srv = (IPrototypeService)parentElement;
					return srv.getAllTopLevelPrototypesCategories().toArray();
				}
				if(parentElement instanceof PrototypeCategory) {
					PrototypeCategory cat = (PrototypeCategory)parentElement;
					return cat.getChildren();
				}
				return null;
			}

		};
	}

	/** {@inheritDoc} */
	@Override
	protected ITableLabelProvider createLabelProvider() {
		return new TableLabelProviderBase() {
			@Override
			public String getColumnText(Object parentElement, int columnIndex) {
				if(parentElement instanceof PrototypeCategory) {
					PrototypeCategory cat = (PrototypeCategory)parentElement;
					return columnIndex == 0 ? cat.getName() : "";
				}
				if(parentElement instanceof Prototype) {
					Prototype proto = (Prototype)parentElement;
					switch(columnIndex) {
						case 0:
							return proto.getName();
						case 1:
							return findProtoypeProviderClassName(proto);
						case 2:
							return proto.getPrototype().getClass().getName();
						case 3:
							return proto.isPrimary() ? "X" : "";
					}
				}
				return "";
			}
		};
	}

	/** {@inheritDoc} */
	@Override
	protected PatternFilter createPatternFilter() {
		return new TableViewerPatternFilter();
	}

	/** {@inheritDoc} */
	@Override
	protected Object getInputObject() {
		return IPrototypeService.INSTANCE;
	}
}
